/*
 *****************************************************************************
 *                                                                           *
 *     (C)  Copyright 2007 Hewlett-Packard Development Company, L.P          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the Free*
 * Software  Foundation; either version 2 of the License, or (at your option)*
 * any later version.                                                        *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY*
 * or FITNESS FOR  A PARTICULAR PURPOSE. See the GNU General Public License  *
 * for more details.                                                         *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 675 Mass Ave, Cambridge, MA 02139, USA.                                   *
 *                                                                           *
 *             										      *
 *                                                                           *
 *                                                                           *
 *****************************************************************************
*/

/*
 *  This program originally derived from and inspired by 
 *  Christophe Varoqui's tur.c, part of libchecker.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include "../cciss_id/cciss_ioctl.h"

#include "checkers.h"

#include "../libmultipath/sg_include.h"

#define TUR_CMD_LEN 6
#define HEAVY_CHECK_COUNT       10

#define MSG_TUR_UP	"cciss_tur checker reports path is up"
#define MSG_TUR_DOWN	"cciss_tur checker reports path is down"
#define MSG_TUR_UNCHECKED 	"cciss_tur checker reports path is unchecked"

struct tur_checker_context {
	void * dummy;
};

int cciss_tur_init (struct checker * c)
{
	return 0;
}

void cciss_tur_free (struct checker * c)
{
	return;
}

extern int
cciss_tur (struct checker * c)
{
        LogvolInfo_struct lvinfo;
	IOCTL_Command_struct cmd;
	memset(&cmd, 0, sizeof(IOCTL_Command_struct));
        cmd.LUN_info.LogDev.Mode = 0x01;

        if (ioctl(c->fd,CCISS_GETLUNINFO,&lvinfo) != 0) {
                MSG(c,MSG_TUR_UNCHECKED);
         	return(PATH_UNCHECKED);
        }

        cmd.LUN_info.LogDev.VolId=lvinfo.LunID&0x3FFFFFFF;
        cmd.Request.Type.Type =  TYPE_CMD;
        cmd.Request.Type.Attribute = ATTR_SIMPLE;
        cmd.Request.Type.Direction = XFER_NONE;
        cmd.buf_size=0;
        cmd.buf=NULL;
        cmd.Request.Timeout = 0;
        cmd.Request.CDBLen = 6;
        cmd.Request.CDB[0] = 0x00;
        cmd.Request.CDB[1] = 0;
        cmd.Request.CDB[2] = 0;
        cmd.Request.CDB[4] = 0;
        cmd.Request.CDB[5] = 0;

        if (ioctl(c->fd, CCISS_PASSTHRU, &cmd) != 0) {
             MSG(c,MSG_TUR_DOWN);
             return(PATH_DOWN);
        }
        if ((cmd.error_info.CommandStatus | cmd.error_info.ScsiStatus )) {
             MSG(c,MSG_TUR_DOWN);
             return(PATH_DOWN);
        }
        MSG(c,MSG_TUR_UP);
        return(PATH_UP);


}
