#ifndef _CONFIG_H
#define _CONFIG_H

#include <sys/types.h>

#define ORIGIN_DEFAULT 0
#define ORIGIN_CONFIG  1

enum devtypes {
	DEV_NONE,
	DEV_DEVT,
	DEV_DEVNODE,
	DEV_DEVMAP
};

struct hwentry {
	char * vendor;
	char * product;
	char * getuid;
	char * getprio;
	char * features;
	char * hwhandler;
	char * selector;
	char * checker_name;

	int pgpolicy;
	int pgfailback;
	int rr_weight;
	int no_path_retry;
	int minio;
	int pg_timeout;
	int flush_on_last_del;
	int fast_io_fail;
	unsigned int dev_loss;
	struct checker * checker;
	char * bl_product;
};

struct mpentry {
	char * wwid;
	char * alias;
	char * getuid;
	char * selector;
	char * getprio;

	int pgpolicy;
	int pgfailback;
	int rr_weight;
	int no_path_retry;
	int minio;
	int pg_timeout;
	int attribute_flags;
	int flush_on_last_del;
	uid_t uid;
	gid_t gid;
	mode_t mode;
};

struct config {
	int verbosity;
	int dry_run;
	int list;
	int pgpolicy_flag;
	int with_sysfs;
	int pgpolicy;
	struct checker * checker;
	enum devtypes dev_type;
	int minio;
	int checkint;
	int max_checkint;
	int pgfailback;
	int remove;
	int rr_weight;
	int no_path_retry;
	int user_friendly_names;
	int pg_timeout;
	int max_fds;
	int force_reload;
	int attribute_flags;
	int flush_on_last_del;
	int queue_without_daemon;
	int checker_timeout;
	int allow_queueing;
	int pg_prio_calc;
	int log_checker_err;
	int fast_io_fail;
	int file_timeout;
	unsigned int dev_loss;
	uid_t uid;
	gid_t gid;
	mode_t mode;

	char * dev;
	char * udev_dir;
	char * selector;
	char * getuid;
	char * getprio;
	char * features;
	char * hwhandler;
	char * bindings_file;

	vector keywords;
	vector mptable;
	vector hwtable;
	vector binvec;
	vector keep_dirs;

	vector blist_devnode;
	vector blist_wwid;
	vector blist_device;
	vector elist_devnode;
	vector elist_wwid;
	vector elist_device;
};

struct config * conf;

extern int push_callout(char * callout);

struct hwentry * find_hwe (vector hwtable, char * vendor, char * product);
struct mpentry * find_mpe (char * wwid);
char * get_mpe_wwid (char * alias);

struct mpentry * alloc_mpe (void);

void free_hwe (struct hwentry * hwe);
void free_hwtable (vector hwtable);
void free_mpe (struct mpentry * mpe);
void free_mptable (vector mptable);

int store_hwe (vector hwtable, struct hwentry *);

int load_config (char * file);
struct config * alloc_config (void);
void free_config (struct config * conf);

#endif
