/******************************************************************************
*******************************************************************************
**
**  Copyright (C) 2009 Red Hat, Inc.  All rights reserved.
**
**  This copyrighted material is made available to anyone wishing to use,
**  modify, copy, or redistribute it subject to the terms and conditions
**  of the GNU General Public License v.2.
**
*******************************************************************************
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>
#include <errno.h>

char *prog;

int
match (char *dev, char *regex_str, char *prio_str, unsigned int *prio)
{
	regex_t regex;
	int err, ret = 0;
	char *errbuf;
	size_t errbuf_size;
	unsigned int prio_match;

	if (sscanf(prio_str, "%u", &prio_match) != 1) {
		fprintf(stderr, "%s: invalid prio '%s'\n", prog, prio_str);
		return 0;
	}
	err = regcomp(&regex, regex_str, REG_EXTENDED|REG_NOSUB);
	if (err){
		errbuf_size = regerror(err, &regex, NULL, 0);
		errbuf = malloc(errbuf_size);
		if (!errbuf) {
			fprintf(stderr, "%s cannot compile regex %s\n", prog,
				regex_str);
			return 0;
		}
		regerror(err, &regex, errbuf, errbuf_size);
		fprintf(stderr, "%s: cannot compile regex %s : %s\n", prog,
			regex_str, errbuf);
		free(errbuf);
		return 0;
	}
	if (regexec(&regex, dev, 0, NULL, 0) == 0){
		*prio = prio_match;
		ret = 1;
	}

	regfree(&regex);
	return ret;
}

int
main (int argc, char *argv[])
{
	unsigned int prio = 0;
	int i;
	size_t regex_str_len = 0;
	char *regex_str;

	prog = argv[0];

	if (argc == 1) {
		fprintf(stderr, "Usage: %s <hbtl|devname> <regex1> <prio1> [<regex2> <prio2>] ...\n", prog);
		fprintf(stderr,
"Warning: This priority callout is based on a path's device name or its\n"
"H:T:B:L. Both of these can change when the node is rebooted, and can\n"
"differ from node to node. (i.e. there is no guarantee that sda will point\n"
"to the same device after a reboot) If you use this callout, it may be\n"
"necessary to manually edit /etc/multipath.conf after any reboot.\n");
	}
	else if (argc < 4 || (argc % 2) != 0) {
		fprintf(stderr, "%s: invalid number of arguments\n", prog);
		fprintf(stderr, "Usage: %s <hbtl|devname> <regex1> <prio1> [<regex2> <prio2>] ...\n", prog);
	}
	else {
		for (i = 2; i < argc; i += 2)
			if (strlen(argv[i]) > regex_str_len)
				regex_str_len = strlen(argv[i]);
		regex_str = malloc(regex_str_len + 3);
		for (i = 2; i < argc; i += 2){
			sprintf(regex_str, "%s%s%s",
				(argv[i][0] == '^')? "" : "^", argv[i],
				(argv[i][strlen(argv[i])-1] == '$')? "" : "$");
			if (match(argv[1], regex_str, argv[i+1], &prio))
				break;
		}
		free(regex_str);
	}

	printf("%u\n", prio);
	exit(0);
}
